#!/usr/bin/env python
# -----------------------------------------------------------------------------------------------------------
# Copyright (c) 2020 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# -----------------------------------------------------------------------------------------------------------
"""
A command line interface to launch the 'Oneshot test' workflow (.github/workflows/oneshot-test.yml).
"""

import os
from pathlib import Path
import subprocess
import re
from pprint import pformat
import textwrap

try:
    import click
    import github

except ImportError:
    raise SystemExit("The script requires PyGithub and click. Please run the following then re-run this script\n\n"
                     "pip install click PyGithub\n")


HERE = Path(__file__).parent


def authenticated_user():
    """Get a logged in Github user."""
    # Anyone know a better way of doing this?
    token = os.environ.get("GITHUB_TOKEN")

    if token is None:
        token = input("Please enter a Github Personal Access Token with at least 'repo/public_repo' scope. "
                      "If you don't have one then create one at https://github.com/settings/tokens.\n"
                      "Alternatively you may set the GITHUB_TOKEN environment variable instead:\n")

    user = github.Github(*token.split(maxsplit=1))
    try:
        user.get_user().login
    except github.BadCredentialsException:
        raise SystemExit("Authentication failed due to invalid credentials.")

    print("Authenticated successfully.")
    return user


def _get_current_branch():
    """Get the branch currently active locally in git.

    The location of this repo is defined based on the location of this script so it is current-working-dir
    independent.
    """
    branch_command = ["git", "-C", str(HERE), "branch"]
    branch_output = subprocess.check_output(branch_command).decode()
    return re.search(r"\*\s*(\S+)\n", branch_output).group(1)


def launch_workflow(workflow, branch, params):
    workflow.update()
    old_len = workflow.get_runs().totalCount

    # Launch the workflow. This only returns a boolean.
    if not workflow.create_dispatch(branch, params):
        raise SystemExit("The workflow failed to launch. Check your authentication token has sufficient permissions to "
                         "use the Github API. It bizarrely requires both read and write access.")

    print("Request has been accepted. Waiting for the build to go live",
          end="")
    while old_len == workflow.get_runs().totalCount:
        print(".", end="")
        workflow.update()
    print()

    build = workflow.get_runs()[0]
    print("Tests are live at:", build.html_url)

    return build


def _norm_comma_space(x):
    """Prettify comma deliminated text to always have one space after a comma."""
    return re.sub(", *", ", ", x)


PYTHONS = ["3.8", "3.9", "3.10", "3.11"]
OSs = ["ubuntu", "windows", "macos"]


@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.argument("package", nargs=-1)
@click.option("--py", multiple=True, default=["3.11"],
              help="Python version(s) to test on. Separate multiple versions with a comma or use this parameter "
                   "multiple times to test multiple versions. You may specify micro versions such as 3.10.9 although "
                   "this is discouraged as they are not guaranteed to be available. Use 'all' to select {}. "
                   "Defaults to '3.11'.".format(PYTHONS))
@click.option("--os", multiple=True, default=["ubuntu"], type=click.Choice(OSs + ["all"], case_sensitive=False),
              help="Which OSs to test on. Use 'all' to specify all three. Defaults to 'ubuntu'.")
@click.option("--fail-fast", default=False, is_flag=True, help="Cancel all other builds if any one of them fails.")
@click.option("--pytest-args", default="", help="Additional arguments to be passed to pytest.")
@click.option("--fork", default=None,
              help="Which fork of pyinstaller-hooks-contrib to use. Defaults to the fork of the authenticated user.")
@click.option("--branch", default=None,
              help="The branch to test. Defaults to using git to get your currently active branch.")
@click.option("--commands", multiple=True,
              help="Additional bash installation commands to run. Ran after setting up Python but before pip-installing"
                   "dependencies.")
@click.option("--browser", default=False, is_flag=True,
              help="Open the live build on Github in a browser window.")
@click.option("--dry-run", is_flag=True, help="Don't launch a build. Just parse and print the parameters.")
def main(package, py, os, fork, branch, pytest_args, fail_fast, commands, browser, dry_run):
    """Launch CI testing of a given package against multiple package or Python versions and OSs.

    The **package** specifies only those to install. Which tests should be ran is inferred implicitly by
    ``@pytest.importorskip``.

    Basic usage: Launch two jobs to test the ``pycparser`` hooks on linux, Pythons 3.10 and 3.11, using the latest
    version of ``pycparser``. And open the build in a browser window.

        python cloud-test.py --py=3.10,3.11 --os=ubuntu --browser pycparser

    The **package** can be anything you'd put on the right hand side of `pip install`. Multiple packages to install can
    be separated by a space: Launch one job which installs and tests two libraries.

        python cloud-test.py pycparser==2.10 passlib==1.7.1

    Multiple versions to be run in separate jobs should be deliminated by a comma: Launch 4 x 2 = 8 jobs to test the
    ``pycparser`` hooks on windows, with all supported Pythons, against two versions of ``pycparser``.

        python cloud-test.py --py=all --os=windows pycparser==2.20, pycparser==2.16

    When you're absolutely certain your hook is ready for it, you may test everything (please use sparingly - this costs
    Github a lot of $$$). This would create 4 x 3 x 3 = 36 jobs.

        python cloud-test.py --py=all --os=all pycparser==2.16, pycparser==2.18, pycparser==2.20

    It costs Github 10x as much to run macOS as it does Linux. So please use Ubuntu as your default OS and test macOS
    last, in conservative batches. Github Actions is free for us but it won't stay that way if we abuse it.

    """

    # --- Parse and normalise input parameters. ---

    # The bulk of the parsing is already done by the workflow. We mostly just need to serialise it into the same format
    # as the web UI.
    if any("all" in i for i in py):
        py = PYTHONS

    if "all" in os:
        os = OSs

    package = " ".join(package)

    params = {
        "python-version": _norm_comma_space(",".join(py)),
        "package": _norm_comma_space(package),
        "os": _norm_comma_space(",".join(os).lower()),
        "pytest_args": pytest_args,
        "fail-fast": str(fail_fast).lower(),
        "commands": "; ".join(commands),
    }

    print("Configuration options to be passed to CI:")
    print(textwrap.indent(pformat(params), "    "))

    # --- Find and connect to the right workflow ---

    #  Login.
    user = authenticated_user()

    # Work out which fork we're supposed to use:
    if fork is None:
        fork = user.get_user().login

    try:
        repo = user.get_repo(fork + "/pyinstaller-hooks-contrib")
    except github.UnknownObjectException:
        raise SystemExit(
            "The repo {}/pyinstaller-hooks-contrib does not exist. Use the --fork option to specify the fork. Or, if "
            "you have adequate permissions, use --fork=pyinstaller to use the master repo.".format(fork))

    # Work out which branch we're using.
    if branch is None:
        try:
            branch = _get_current_branch()
        except subprocess.SubprocessError:
            raise SystemExit(
                "Failed to guess the branch using git. Please specify it manually using the --branch option.")

    print("Using the '{}' branch of:\n    {}".format(branch, repo.html_url))

    branch = repo.get_branch(branch)

    # Get the workflow to trigger:

    # There doesn't seem to be a better way to get a specific workflow besides
    # iterating through them until we find the right one.
    workflow = next(i for i in repo.get_workflows()
                    if i.name == "Oneshot test")

    # --- Launch the build ---

    if dry_run:
        print("Dry run only: abort")
        return

    build = launch_workflow(workflow, branch, params)

    if browser:
        import webbrowser
        webbrowser.open(build.html_url)


if __name__ == "__main__":
    main()



# ------------------------------------------------------------------
# Copyright (c) 2020 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------
"""
Hook for PyOpenGL 3.x versions from 3.0.0b6 up. Previous versions have a
plugin system based on pkg_resources which is problematic to handle correctly
under pyinstaller; 2.x versions used to run fine without hooks, so this one
shouldn't hurt.
"""

from PyInstaller.compat import is_win, is_darwin
from PyInstaller.utils.hooks import collect_data_files, collect_submodules


# PlatformPlugin performs a conditional import based on os.name and
# sys.platform. PyInstaller misses this so let's add it ourselves...
if is_win:
    hiddenimports = ['OpenGL.platform.win32']
elif is_darwin:
    hiddenimports = ['OpenGL.platform.darwin']
# Use glx for other platforms (Linux, ...)
else:
    hiddenimports = ['OpenGL.platform.glx']

# Arrays modules are needed too.
hiddenimports += collect_submodules('OpenGL.arrays')

# PyOpenGL 3.x uses ctypes to load DLL libraries. PyOpenGL windows installer
# adds necessary dll files to
#   DLL_DIRECTORY = os.path.join( os.path.dirname( OpenGL.__file__ ), 'DLLS')
# PyInstaller is not able to find these dlls. Just include them all as data
# files.
if is_win:
    datas = collect_data_files('OpenGL')



# ------------------------------------------------------------------
# Copyright (c) 2020 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------
"""
Hook for cryptography module from the Python Cryptography Authority.
"""

import os
import glob
import pathlib

from PyInstaller import compat
from PyInstaller import isolated
from PyInstaller.utils.hooks import (
    collect_submodules,
    copy_metadata,
    get_module_file_attribute,
    is_module_satisfies,
    logger,
)

# get the package data so we can load the backends
datas = copy_metadata('cryptography')

# Add the backends as hidden imports
hiddenimports = collect_submodules('cryptography.hazmat.backends')

# Add the OpenSSL FFI binding modules as hidden imports
hiddenimports += collect_submodules('cryptography.hazmat.bindings.openssl') + ['_cffi_backend']


# Include the cffi extensions as binaries in a subfolder named like the package.
# The cffi verifier expects to find them inside the package directory for
# the main module. We cannot use hiddenimports because that would add the modules
# outside the package.
# NOTE: this is not true anymore with PyInstaller >= 6.0, but we keep it like this for compatibility with 5.x series.
binaries = []
cryptography_dir = os.path.dirname(get_module_file_attribute('cryptography'))
for ext in compat.EXTENSION_SUFFIXES:
    ffimods = glob.glob(os.path.join(cryptography_dir, '*_cffi_*%s*' % ext))
    for f in ffimods:
        binaries.append((f, 'cryptography'))


# Check if `cryptography` is dynamically linked against OpenSSL >= 3.0.0. In that case, we might need to collect
# external OpenSSL modules, if OpenSSL was built with modules support. It seems the best indication of this is the
# presence of `ossl-modules` directory next to the OpenSSL shared library.
#
# NOTE: PyPI wheels ship with extensions statically linked against OpenSSL, so this is mostly catering alternative
# installation methods (Anaconda on all OSes, Homebrew on macOS, various linux distributions).
try:
    @isolated.decorate
    def _check_cryptography_openssl3():
        # Check if OpenSSL 3 is used.
        from cryptography.hazmat.backends.openssl.backend import backend
        openssl_version = backend.openssl_version_number()
        if openssl_version < 0x30000000:
            return False, None

        # Obtain path to the bindings module for binary dependency analysis. Under older versions of cryptography,
        # this was a separate `_openssl` module; in contemporary versions, it is `_rust` module.
        try:
            import cryptography.hazmat.bindings._openssl as bindings_module
        except ImportError:
            import cryptography.hazmat.bindings._rust as bindings_module

        return True, str(bindings_module.__file__)

    uses_openssl3, bindings_module = _check_cryptography_openssl3()
except Exception:
    logger.warning(
        "hook-cryptography: failed to determine whether cryptography is using OpenSSL >= 3.0.0", exc_info=True
    )
    uses_openssl3, bindings_module = False, None

if uses_openssl3:
    # Determine location of OpenSSL shared library, provided that extension module is dynamically linked against it.
    # This requires the new PyInstaller.bindepend API from PyInstaller >= 6.0.
    openssl_lib = None
    if is_module_satisfies("PyInstaller >= 6.0"):
        from PyInstaller.depend import bindepend

        if compat.is_win:
            SSL_LIB_NAME = 'libssl-3-x64.dll' if compat.is_64bits else 'libssl-3.dll'
        elif compat.is_darwin:
            SSL_LIB_NAME = 'libssl.3.dylib'
        else:
            SSL_LIB_NAME = 'libssl.so.3'

        linked_libs = bindepend.get_imports(bindings_module)
        openssl_lib = [
            # Compare the basename of lib_name, because lib_fullpath is None if we fail to resolve the library.
            lib_fullpath for lib_name, lib_fullpath in linked_libs if os.path.basename(lib_name) == SSL_LIB_NAME
        ]
        openssl_lib = openssl_lib[0] if openssl_lib else None
    else:
        logger.warning(
            "hook-cryptography: full support for cryptography + OpenSSL >= 3.0.0 requires PyInstaller >= 6.0"
        )

    # Check for presence of ossl-modules directory next to the OpenSSL shared library.
    if openssl_lib:
        logger.info("hook-cryptography: cryptography uses dynamically-linked OpenSSL: %r", openssl_lib)

        openssl_lib_dir = pathlib.Path(openssl_lib).parent

        # Collect whole ossl-modules directory, if it exists.
        ossl_modules_dir = openssl_lib_dir / 'ossl-modules'

        # Msys2/MinGW installations on Windows put the shared library into `bin` directory, but the modules are
        # located in `lib` directory. Account for that possibility.
        if not ossl_modules_dir.is_dir() and openssl_lib_dir.name == 'bin':
            ossl_modules_dir = openssl_lib_dir.parent / 'lib' / 'ossl-modules'

        # On Alpine linux, the true location of shared library is /lib directory, but the modules' directory is located
        # in /usr/lib instead. Account for that possibility.
        if not ossl_modules_dir.is_dir() and openssl_lib_dir == pathlib.Path('/lib'):
            ossl_modules_dir = pathlib.Path('/usr/lib/ossl-modules')

        if ossl_modules_dir.is_dir():
            logger.debug("hook-cryptography: collecting OpenSSL modules directory: %r", str(ossl_modules_dir))
            binaries.append((str(ossl_modules_dir), 'ossl-modules'))
    else:
        logger.info("hook-cryptography: cryptography does not seem to be using dynamically linked OpenSSL.")



# ------------------------------------------------------------------
# Copyright (c) 2026 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.utils.hooks import is_module_satisfies

hiddenimports = []
datas = []

# Starting with v7.0.0, we need to collect mypy extension, and the data files from chardet/models
if is_module_satisfies("chardet >= 7.0.0"):
    from PyInstaller.utils.hooks import collect_data_files
    from _pyinstaller_hooks_contrib.utils.mypy import find_mypyc_module_for_dist

    hiddenimports += find_mypyc_module_for_dist('chardet')
    datas += collect_data_files('chardet')

# In the Windows PyPI wheel of chardet v7.4.0post2, the extensions in `chardet.pipeline` seem to come with individual
# `__mypyc` counterparts (for example the `chardet.pipeline.orchestrator` extension references a corresponding
# `chardet.pipeline.orchestrator__mypyc` extension). Ensure all these are collected by collecting submodules.
if is_module_satisfies("chardet >= 7.4.0"):
    from PyInstaller.utils.hooks import collect_submodules
    hiddenimports += collect_submodules('chardet.pipeline')



# ------------------------------------------------------------------
# Copyright (c) 2024 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

# https://github.com/enthought/comtypes/blob/1.4.5/comtypes/client/_generate.py#L271-L280
hiddenimports = [
    "comtypes.persist",
    "comtypes.typeinfo",
    "comtypes.automation",
    "comtypes.stream",
    "comtypes",
    "ctypes.wintypes",
    "ctypes",
]



# ------------------------------------------------------------------
# Copyright (c) 2024 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.utils.hooks import is_module_satisfies, collect_data_files

hiddenimports = []
datas = []

# In pydicom 3.0.0, the `pydicom.encoders` plugins were renamed to `pydicom.pixels.encoders`, and
# `pydicom.pixels.decoders` were also added. We need to collect them all, because they are loaded during
# `pydicom` module initialization. We intentionally avoid using `collect_submodules` here, because that causes
# import of `pydicom` with logging framework initialized, which results in error tracebacks being logged for all plugins
# with missing libraries (see https://github.com/pydicom/pydicom/issues/2128).
if is_module_satisfies('pydicom >= 3.0.0'):
    hiddenimports += [
        "pydicom.pixels.decoders.gdcm",
        "pydicom.pixels.decoders.pylibjpeg",
        "pydicom.pixels.decoders.pillow",
        "pydicom.pixels.decoders.pyjpegls",
        "pydicom.pixels.decoders.rle",
        "pydicom.pixels.encoders.gdcm",
        "pydicom.pixels.encoders.pylibjpeg",
        "pydicom.pixels.encoders.native",
        "pydicom.pixels.encoders.pyjpegls",
    ]

    # With pydicom 3.0.0, initialization of `pydicom` (unnecessarily) imports `pydicom.examples`, which attempts to set
    # up several test datasets: https://github.com/pydicom/pydicom/blob/v3.0.0/src/pydicom/examples/__init__.py#L10-L24
    # Some of those are bundled with the package itself, some are downloaded (into `.pydicom/data` directory in user's
    # home directory) on he first `pydicom.examples` import.
    #
    # The download code requires `pydicom/data/urls.json` and `pydicom/data/hashes.json`; the lack of former results in
    # run-time error, while the lack of latter results in warnings about dataset download failure.
    #
    # The test data files that are bundled with the package are not listed in `urls.json`, so if they are missing, there
    # is not attempt to download them. Therefore, try to get away without collecting them here - if anyone actually
    # requires them in the frozen application, let them explicitly collect them.
    additional_data_patterns = [
        'urls.json',
        'hashes.json',
    ]
else:
    hiddenimports += [
        "pydicom.encoders.gdcm",
        "pydicom.encoders.pylibjpeg",
        "pydicom.encoders.native",
    ]
    additional_data_patterns = []

# Collect data files from `pydicom.data`; charset files and palettes might be needed during processing, so always
# collect them. Some other data files became required in v3.0.0 - the corresponding patterns are set accordingly in
# `additional_data_patterns` in the above if/else block.
datas += collect_data_files(
    'pydicom.data',
    includes=[
        'charset_files/*',
        'palettes/*',
        *additional_data_patterns,
    ],
)



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

import os
import pathlib
import platform

from PyInstaller.utils.hooks import get_package_paths, logger


# tkinterdnd2 contains a tkdnd sub-directory which contains platform-specific directories with shared library and .tcl
# files. Collect only the relevant directory, by matching the decision logic from:
# https://github.com/Eliav2/tkinterdnd2/blob/9a55907e430234bf8ab72ea614f84af9cc89598c/tkinterdnd2/TkinterDnD.py#L33-L51
def _collect_platform_subdir(system, machine):
    datas = []
    binaries = []

    # Under Windows, `platform.machine()` returns the identifier of the *host* architecture, which does not necessarily
    # match the architecture of the running process (for example, when running x86 process under x64 Windows, or when
    # running either x86 or x64 process under arm64 Windows). The architecture of the running process can be obtained
    # from the `PROCESSOR_ARCHITECTURE` environment variable, which is automatically set by Windows / WOW64 subsystem.
    #
    # NOTE: at the time of writing (tkinterdnd2 v0.4.2), tkinterdnd2 does not account for this, and attempts to load
    # the shared library from incorrect directory; as this fails due to architecture mismatch, there is no point in
    # us trying to collect that (incorrect) directory.
    if system == "Windows":
        machine = os.environ.get("PROCESSOR_ARCHITECTURE", machine)

    # Resolve the platform-specific sub-directory name and shared library suffix.
    DIR_NAMES = {
        "Darwin": {
            "arm64": "osx-arm64",
            "x86_64": "osx-x64",
        },
        "Linux": {
            "aarch64": "linux-arm64",
            "x86_64": "linux-x64",
        },
        "Windows": {
            "ARM64": "win-arm64",
            "AMD64": "win-x64",
            "x86": "win-x86",
        }
    }
    dir_name = DIR_NAMES.get(system, {}).get(machine, None)

    LIB_SUFFICES = {
        "Darwin": "*.dylib",
        "Linux": "*.so",
        "Windows": "*.dll",
    }
    lib_suffix = LIB_SUFFICES.get(system, None)

    if dir_name is None or lib_suffix is None:
        logger.warning(
            "hook-tkinterdnd2: unsupported platform (%s, %s)! Platform-specific directory will not be collected!",
            system, machine
        )
        return datas, binaries

    pkg_base, pkg_dir = get_package_paths("tkinterdnd2")

    dest_dir = os.path.join("tkinterdnd2", "tkdnd", dir_name)
    src_path = pathlib.Path(pkg_dir) / "tkdnd" / dir_name

    if not src_path.is_dir():
        logger.warning("hook-tkinterdnd2: platform-specific sub-directory %r does not exist!", str(src_path))
        return datas, binaries

    # Collect the shared library.
    for entry in src_path.glob(lib_suffix):
        binaries.append((str(entry), dest_dir))

    # Collect the .tcl files.
    for entry in src_path.glob("*.tcl"):
        datas.append((str(entry), dest_dir))

    return datas, binaries


datas, binaries = _collect_platform_subdir(platform.system(), platform.machine())



# ------------------------------------------------------------------
# Copyright (c) 2021 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------
import os
import pathlib
import shutil

from PyInstaller import compat
from PyInstaller.depend import bindepend
from PyInstaller.utils.hooks import logger


def _collect_graphviz_files():
    binaries = []
    datas = []

    # A working `pygraphviz` installation requires graphviz programs in PATH. Attempt to resolve the `dot` executable to
    # see if this is the case.
    dot_binary = shutil.which('dot')
    if not dot_binary:
        logger.warning(
            "hook-pygraphviz: 'dot' program not found in PATH!"
        )
        return binaries, datas
    logger.info("hook-pygraphviz: found 'dot' program: %r", dot_binary)
    bin_dir = pathlib.Path(dot_binary).parent

    # Collect graphviz programs that might be called from `pygaphviz.agraph.AGraph`:
    # https://github.com/pygraphviz/pygraphviz/blob/pygraphviz-1.14/pygraphviz/agraph.py#L1330-L1348
    # On macOS and on Linux, several of these are symbolic links to a single executable.
    progs = (
        "neato",
        "dot",
        "twopi",
        "circo",
        "fdp",
        "nop",
        "osage",
        "patchwork",
        "gc",
        "acyclic",
        "gvpr",
        "gvcolor",
        "ccomps",
        "sccmap",
        "tred",
        "sfdp",
        "unflatten",
    )

    logger.debug("hook-pygraphviz: collecting graphviz program executables...")
    for program_name in progs:
        program_binary = shutil.which(program_name)
        if not program_binary:
            logger.debug("hook-pygaphviz: graphviz program %r not found!", program_name)
            continue

        # Ensure that the program executable was found in the same directory as the `dot` executable. This should
        # prevent us from falling back to other graphviz installations that happen to be in PATH.
        if pathlib.Path(program_binary).parent != bin_dir:
            logger.debug(
                "hook-pygraphviz: found program %r (%r) outside of directory %r - ignoring!",
                program_name, program_binary, str(bin_dir)
            )
            continue

        logger.debug("hook-pygraphviz: collecting graphviz program %r: %r", program_name, program_binary)
        binaries += [(program_binary, '.')]

    # Graphviz shared libraries should be automatically collected when PyInstaller performs binary dependency
    # analysis of the collected program executables as part of the main build process. However, we need to manually
    # collect plugins and their accompanying config file.
    logger.debug("hook-pygraphviz: looking for graphviz plugin directory...")
    if compat.is_win:
        # Under Windows, we have several installation variants:
        #  - official installers and builds from https://gitlab.com/graphviz/graphviz/-/releases
        #  - chocolatey
        #  - msys2
        #  - Anaconda
        # In all variants, the plugins and the config file are located in the `bin` directory, next to the program
        # executables.
        plugin_dir = bin_dir
        plugin_dest_dir = '.'  # Collect into top-level application directory.
        # Official builds and Anaconda use unversioned `gvplugin-{name}.dll` plugin names, while msys2 uses
        # versioned `libgvplugin-{name}-{version}.dll` plugin names (with "lib" prefix).
        plugin_pattern = '*gvplugin*.dll'
    else:
        # Perform binary dependency analysis on the `dot` executable to obtain the path to graphiz shared libraries.
        # These need to be in the library search path for the programs to work, or discoverable via run-paths
        # (e.g., Anaconda on Linux and macOS, Homebrew on macOS).
        graphviz_lib_candidates = ['cdt', 'gvc', 'cgraph']

        if hasattr(bindepend, 'get_imports'):
            # PyInstaller >= 6.0
            dot_imports = [path for name, path in bindepend.get_imports(dot_binary) if path is not None]
        else:
            # PyInstaller < 6.0
            dot_imports = bindepend.getImports(dot_binary)

        graphviz_lib_paths = [
            path for path in dot_imports
            if any(candidate in os.path.basename(path) for candidate in graphviz_lib_candidates)
        ]

        if not graphviz_lib_paths:
            logger.warning("hook-pygraphviz: could not determine location of graphviz shared libraries!")
            return binaries, datas

        graphviz_lib_dir = pathlib.Path(graphviz_lib_paths[0]).parent
        logger.debug("hook-pygraphviz: location of graphviz shared libraries: %r", str(graphviz_lib_dir))

        # Plugins should be located in `graphviz` directory next to shared libraries.
        plugin_dir = graphviz_lib_dir / 'graphviz'
        plugin_dest_dir = 'graphviz'  # Collect into graphviz sub-directory.

        if compat.is_darwin:
            plugin_pattern = '*gvplugin*.dylib'
        else:
            # Collect only versioned .so library files (for example, `/lib64/graphviz/libgvplugin_core.so.6` and
            # `/lib64/graphviz/libgvplugin_core.so.6.0.0`; the former usually being a symbolic link to the latter).
            # The unversioned .so library files (such as `lib64/graphviz/libgvplugin_core.so`), if available, are
            # meant for linking (and are usually installed as part of development package).
            plugin_pattern = '*gvplugin*.so.*'

    if not plugin_dir.is_dir():
        logger.warning("hook-pygraphviz: could not determine location of graphviz plugins!")
        return binaries, datas

    logger.info("hook-pygraphviz: collecting graphviz plugins from directory: %r", str(plugin_dir))

    binaries += [(str(file), plugin_dest_dir) for file in plugin_dir.glob(plugin_pattern)]
    datas += [(str(file), plugin_dest_dir) for file in plugin_dir.glob("config*")]  # e.g., `config6`

    return binaries, datas


binaries, datas = _collect_graphviz_files()



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------
from PyInstaller.utils.hooks import collect_submodules, collect_data_files
from _pyinstaller_hooks_contrib.utils.mypy import find_mypyc_module_for_dist

hiddenimports = [
    # `black` (or rather, its `blib2to3` library) uses `mypy`, and includes a top-level module with
    # dynamically-generated name prefix; for example, `30fcd23745efe32ce681__mypyc`.
    *find_mypyc_module_for_dist('black'),
    'dataclasses',
    'pkgutil',
    'tempfile',
    *collect_submodules('blib2to3')
]

# Ensure that data files, such as `PatternGrammar.txt` and `Grammar.txt`, are collected.
datas = collect_data_files('blib2to3')



# ------------------------------------------------------------------
# Copyright (c) 2026 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from _pyinstaller_hooks_contrib.compat import importlib_metadata


# Given the distribution name, find the top-level `mypyc` extension module belonging to that distribution. The said
# top-level module is called something like `30fcd23745efe32ce681__mypyc`; the prefix changes across different versions
# of the same distribution (and, of course, across different distributions). Therefore, we need to obtain the name by
# looking at distribution's list of files.
def find_mypyc_module_for_dist(dist_name):
    try:
        dist = importlib_metadata.distribution(dist_name)
    except importlib_metadata.PackageNotFoundError:
        return []
    return [entry.name.split('.')[0] for entry in (dist.files or []) if '__mypyc' in entry.name]



import sys
import argparse

import toga


class TestApp(toga.App):
    def __init__(self, *args, automatic_shutdown=0, **kwargs):
        # Hack: on macOS, toga assumes that the only possible build type is .app bundle, and tries to fall back to .app
        # bundles icon instead of default. So until this is fixed on toga's side, we need to explicitly specify the icon
        # if we are running in a POSIX build.
        if getattr(sys, "frozen", False) and sys.platform == 'darwin':
            is_app_bundle = (
                sys._MEIPASS.endswith("Contents/Frameworks") or  # PyInstaller >= 6.0
                sys._MEIPASS.endswith("Contents/MacOS")  # < PyInstaller < 6.0
            )
            if not is_app_bundle:
                print("Explicitly specifying icon for macOS POSIX build...")
                kwargs["icon"] = toga.icons.Icon.DEFAULT_ICON

        super().__init__(*args, **kwargs)
        self._automatic_shutdown = automatic_shutdown

    def startup(self):
        print("Building application UI...", file=sys.stderr)

        # Create simple UI with label and a button
        box = toga.Box()
        box.style.update(direction=toga.style.pack.COLUMN, padding=10)

        label = toga.Label("Hello world!", style=toga.style.pack.Pack(text_align=toga.style.pack.CENTER))
        box.add(label)

        button = toga.Button("Test button", on_press=self.on_button_press)
        box.add(button)

        self.main_window = toga.MainWindow()
        self.main_window.content = box
        self.main_window.show()

    def on_button_press(self, widget):
        print("Button pressed!", file=sys.stderr)

    def on_running(self):
        print("Application running!", file=sys.stderr)

        # Schedule automatic shutdown
        if self._automatic_shutdown > 0:
            print(f"Requesting shut down in {self._automatic_shutdown:.2f} seconds...", file=sys.stderr)
            self.loop.call_later(self._automatic_shutdown, self.request_exit)

    def on_exit(self):
        print("Application is about to exit!", file=sys.stderr)
        return True


def main():
    parser = argparse.ArgumentParser(description='Test application.')
    parser.add_argument(
        '--automatic-shutdown',
        metavar='seconds',
        type=float,
        default=0,
        help='Automatically shut down the application after specified number of seconds.',
    )
    args = parser.parse_args()

    app = TestApp("Test app", "org.pyinstaller.toga.test-app", automatic_shutdown=args.automatic_shutdown)
    print("Entering main loop...", file=sys.stderr)
    app.main_loop()
    print("Exited main loop", file=sys.stderr)


if __name__ == "__main__":
    main()



# ------------------------------------------------------------------
# Copyright (c) 2024 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.utils.hooks import collect_data_files, is_module_satisfies

# This was required with langchain < 1.0.0; in contemporary versions, the package does not contain any data files,
# so this should be effectively a no-op.
datas = collect_data_files('langchain')

# In langchain 1.2.1 the import logic for optional add-on packages was refactored, and direct imports in conditional
# blocks were replaced with dictionary-based look-up; see https://github.com/langchain-ai/langchain/pull/32813
# So now we need to use that same dictionary to explicitly collect the optional packages that happen to be installed
# in the build environment.
if is_module_satisfies('langchain >= 1.2.1'):
    from PyInstaller import isolated

    @isolated.decorate
    def get_optional_packages(var_name):
        packages = set()

        try:
            import langchain.chat_models.base
            providers = getattr(langchain.chat_models.base, var_name)
            packages.update(package_name for package_name, *_ in providers.values())
        except Exception:
            pass

        try:
            import langchain.embeddings.base
            providers = getattr(langchain.embeddings.base, var_name)
            packages.update(package_name for package_name, *_ in providers.values())
        except Exception:
            pass

        return sorted(packages)

    # langchain 1.2.10 renamed the `_SUPPORTED_PROVIDERS` into `_BUILTIN_PROVIDERS`.
    if is_module_satisfies('langchain >= 1.2.10'):
        var_name = '_BUILTIN_PROVIDERS'
    else:
        var_name = '_SUPPORTED_PROVIDERS'

    hiddenimports = get_optional_packages(var_name)
    warn_on_missing_hiddenimports = False



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller import compat
from PyInstaller.utils.hooks import collect_dynamic_libs, collect_data_files

if compat.is_win:
    # On Windows, pymeshlab wheel seems to include Qt libraries and plugins (and their dependencies, as well as meshlab
    # plugins), as well as some of data files (such as translation files); all in the top-level package directory.
    binaries = collect_dynamic_libs('pymeshlab')
    datas = collect_data_files('pymeshlab')
elif compat.is_linux:
    # On linux, only Qt libraries and plugins (and their dependencies, as well as meshlab plugins) seem to be included;
    # all in lib subdir.
    # NOTE: collect_dynamic_libs() does not collect versioned .so files; those are picked up by collect_data_files().
    binaries = collect_dynamic_libs('pymeshlab.lib') + collect_data_files('pymeshlab.lib')
elif compat.is_darwin:
    # On macOS, Frameworks sub-directory contains Qt .framework bundles and other shared libraries, all of which seem
    # to be picked up by binary dependency analysis. Avoid explicitly collecting anything from there to avoid
    # interfering with PyInstaller's attempts to fix-up .framework bundle structure (see pyinstaller/pyinstaller#9335).
    # The plugins for both meshlab and Qt are in PlugIns directory, and we need to collect those.
    binaries = collect_dynamic_libs('pymeshlab.PlugIns')

# On linux, prevent binary dependency analysis from generating symbolic links for bundles shared libraries to the
# top-level application directory. For some god-forsaken reason, the wheel includes a copy of python shared library,
# which might actually be ABI incompatible with version of python that frozen application is built with (for example,
# libpython3.13.so.1.0 shipped with pymeshlab 2025.7 causes `undefined symbol: _Py_GetExecutor` in `_opcode` extension
# collected from Fedora-provided python 3.13.11).
if compat.is_linux:
    bindepend_symlink_suppression = [
        '**/pymeshlab/lib/*',
    ]



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.utils.hooks import copy_metadata, is_module_satisfies

# duckdb requires stdlib `inspect` module. On newer python versions (>= 3.10), it is collected as a dependency of other
# stdlib modules, while on older python versions this is not the case. Therefore, we add it to hidden imports regardless
# of python version.
hiddenimports = ['inspect']

# Starting with v1.4.0, `duckdb` uses `importlib.metadata.version()` to determine its version.
if is_module_satisfies("duckdb >= 1.4.0"):
    datas = copy_metadata('duckdb')



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

import pytest

from PyInstaller import isolated
from PyInstaller.utils.hooks import can_import_module
from PyInstaller.utils.tests import importorskip


# Run the tests in onedir mode only
pytestmark = pytest.mark.parametrize('pyi_builder', ['onedir'], indirect=True)


@isolated.decorate
def _list_all_vtkmodules_submodules():
    try:
        import vtkmodules
    except Exception:
        return []

    return sorted(list(vtkmodules.__all__))


# Basic import tests for sub-modules of vtkmodules. Run only on demand, and only in onedir mode.
@pytest.mark.slow
@importorskip('vtkmodules')
@pytest.mark.parametrize('submodule', _list_all_vtkmodules_submodules())
def test_vtkmodules(pyi_builder, submodule):
    modname = f"vtkmodules.{submodule}"
    if not can_import_module(modname):
        pytest.skip(f"Module '{modname}' cannot be imported.")
    pyi_builder.test_source(f"""
        import {modname}
        """)



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

import sys
from PyInstaller.utils.hooks import can_import_module, copy_metadata, is_module_satisfies

# Starting with narwhals 1.35.0, we need to collect metadata for `typing_extensions` if the module is available.
# The codepath that checks metadata for `typing_extensions` is not executed under python >= 3.13, so we can avoid
# collection there.
datas = []
if sys.version_info < (3, 13):  # PyInstaller.compat.is_py313 is available only in PyInstaller >= 6.10.0.
    if is_module_satisfies("narwhals >= 1.35.0") and can_import_module("typing_extensions"):
        datas += copy_metadata("typing_extensions")



import os

from _pyinstaller_hooks_contrib.compat import importlib_metadata
import packaging.version

try:
    vtk_version = packaging.version.Version(importlib_metadata.version('vtk')).release
except Exception:
    vtk_version = None


# This list of dependencies was obtained via analysis based on code in `vtkmodules/generate_pyi.py` and augmented with
# missing entries until all tests from `test_vtkmodules` pass. Instead of a pre-computed list, we could dynamically
# analyze each module when the hook is executed; however, such approach would be slower, and would also not account
# for all dependencies that had to be added manually.
#
# NOTE: `vtkmodules.vtkCommonCore` is a dependency of every module, so do not list it here. Modules with no additional
# dependencies are also not listed.
_module_dependencies = {
    'vtkmodules.vtkAcceleratorsVTKmDataModel': [
        'vtkmodules.vtkAcceleratorsVTKmCore',
        'vtkmodules.vtkCommonDataModel',
    ],
    'vtkmodules.vtkAcceleratorsVTKmFilters': [
        'vtkmodules.vtkAcceleratorsVTKmCore',
        'vtkmodules.vtkAcceleratorsVTKmDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCore',
        'vtkmodules.vtkFiltersGeneral',
        'vtkmodules.vtkFiltersGeometry',
        'vtkmodules.vtkImagingCore',
    ],
    'vtkmodules.vtkChartsCore': [
        'vtkmodules.vtkRenderingContext2D',
        'vtkmodules.vtkFiltersGeneral',
    ],
    'vtkmodules.vtkCommonColor': [
        'vtkmodules.vtkCommonDataModel',
    ],
    'vtkmodules.vtkCommonComputationalGeometry': [
        'vtkmodules.vtkCommonDataModel',
    ],
    'vtkmodules.vtkCommonDataModel': [
        'vtkmodules.vtkCommonMath',
        'vtkmodules.vtkCommonTransforms',
        'vtkmodules.util.data_model',
    ],
    'vtkmodules.vtkCommonExecutionModel': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.util.execution_model',
    ],
    'vtkmodules.vtkCommonMisc': [
        'vtkmodules.vtkCommonMath',
    ],
    'vtkmodules.vtkCommonTransforms': [
        'vtkmodules.vtkCommonMath',
    ],
    'vtkmodules.vtkDomainsChemistry': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOLegacy',
        'vtkmodules.vtkIOXMLParser',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkDomainsChemistryOpenGL2': [
        'vtkmodules.vtkDomainsChemistry',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkFiltersAMR': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersCellGrid': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersCore': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkCommonMisc',
    ],
    'vtkmodules.vtkFiltersExtraction': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersGeneral',
    ],
    'vtkmodules.vtkFiltersFlowPaths': [
        'vtkmodules.vtkCommonComputationalGeometry',
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkCommonMath',
    ],
    'vtkmodules.vtkFiltersGeneral': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCore',
    ],
    'vtkmodules.vtkFiltersGeneric': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersGeometry': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersGeometryPreview': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersHybrid': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkCommonTransforms',
        'vtkmodules.vtkFiltersGeometry',
    ],
    'vtkmodules.vtkFiltersHyperTree': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCore',
        'vtkmodules.vtkFiltersGeneral',
    ],
    'vtkmodules.vtkFiltersImaging': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersStatistics',
    ],
    'vtkmodules.vtkFiltersModeling': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersGeneral',
    ],
    'vtkmodules.vtkFiltersParallel': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCore',
        'vtkmodules.vtkFiltersExtraction',
        'vtkmodules.vtkFiltersGeneral',
        'vtkmodules.vtkFiltersGeometry',
        'vtkmodules.vtkFiltersHybrid',
        'vtkmodules.vtkFiltersHyperTree',
        'vtkmodules.vtkFiltersModeling',
        'vtkmodules.vtkFiltersSources',
        'vtkmodules.vtkFiltersTexture',
    ],
    'vtkmodules.vtkFiltersParallelDIY2': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCore',
        'vtkmodules.vtkFiltersParallel',
    ],
    'vtkmodules.vtkFiltersParallelImaging': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersImaging',
        'vtkmodules.vtkFiltersParallel',
        'vtkmodules.vtkImagingCore',
    ],
    'vtkmodules.vtkFiltersParallelStatistics': [
        'vtkmodules.vtkFiltersStatistics',
    ],
    'vtkmodules.vtkFiltersPoints': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersModeling',
    ],
    'vtkmodules.vtkFiltersProgrammable': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersPython': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersReduction': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersSMP': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkFiltersCore',
        'vtkmodules.vtkFiltersGeneral',
    ],
    'vtkmodules.vtkFiltersSelection': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersSources': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersStatistics': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersTemporal': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCore',
    ],
    'vtkmodules.vtkFiltersTensor': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersTexture': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersTopology': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkFiltersVerdict': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkGeovisCore': [
        'vtkmodules.vtkCommonTransforms',
    ],
    'vtkmodules.vtkIOAMR': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOAsynchronous': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
        'vtkmodules.vtkIOImage',
        'vtkmodules.vtkIOXML',
    ],
    'vtkmodules.vtkIOAvmesh': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOCGNSReader': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOCONVERGECFD': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOCellGrid': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCellGrid',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOCesium3DTiles': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOChemistry': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOCityGML': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOCore': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOERF': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOEnSight': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOEngys': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOExodus': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
        'vtkmodules.vtkIOXMLParser',
    ],
    'vtkmodules.vtkIOExport': [
        'vtkmodules.vtkIOCore',
        'vtkmodules.vtkIOImage',
        'vtkmodules.vtkIOXML',
        'vtkmodules.vtkRenderingContext2D',
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingFreeType',
        'vtkmodules.vtkRenderingVtkJS',
    ],
    'vtkmodules.vtkIOExportGL2PS': [
        'vtkmodules.vtkIOExport',
        'vtkmodules.vtkRenderingGL2PSOpenGL2',
    ],
    'vtkmodules.vtkIOExportPDF': [
        'vtkmodules.vtkIOExport',
        'vtkmodules.vtkRenderingContext2D',
    ],
    'vtkmodules.vtkIOFDS': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOFLUENTCFF': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOGeoJSON': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOGeometry': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
        'vtkmodules.vtkIOLegacy',
    ],
    'vtkmodules.vtkIOH5Rage': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOH5part': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOHDF': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOIOSS': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersCellGrid',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOImport': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkIOImage': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkImagingCore',
    ],
    'vtkmodules.vtkIOInfovis': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOLegacy',
        'vtkmodules.vtkIOXML',
    ],
    'vtkmodules.vtkIOLANLX3D': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOLSDyna': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOXMLParser',
    ],
    'vtkmodules.vtkIOLegacy': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCellGrid',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOMINC': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
        'vtkmodules.vtkIOImage',
    ],
    'vtkmodules.vtkIOMotionFX': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOMovie': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIONetCDF': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOOMF': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOOggTheora': [
        'vtkmodules.vtkIOMovie',
    ],
    'vtkmodules.vtkIOPIO': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOPLY': [
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOParallel': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
        'vtkmodules.vtkIOGeometry',
        'vtkmodules.vtkIOImage',
        'vtkmodules.vtkIOLegacy',
    ],
    'vtkmodules.vtkIOParallelExodus': [
        'vtkmodules.vtkIOExodus',
    ],
    'vtkmodules.vtkIOParallelLSDyna': [
        'vtkmodules.vtkIOLSDyna',
    ],
    'vtkmodules.vtkIOParallelXML': [
        'vtkmodules.vtkIOXML',
    ],
    'vtkmodules.vtkIOSQL': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOCore',
    ],
    'vtkmodules.vtkIOSegY': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOImage',
    ],
    'vtkmodules.vtkIOTRUCHAS': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOTecplotTable': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOVPIC': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOVeraOut': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOVideo': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkIOXML': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOXMLParser',
    ],
    'vtkmodules.vtkIOXMLParser': [
        'vtkmodules.vtkCommonDataModel',
    ],
    'vtkmodules.vtkIOXdmf2': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkIOLegacy',
    ],
    'vtkmodules.vtkImagingColor': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkImagingCore',
    ],
    'vtkmodules.vtkImagingCore': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkImagingFourier': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkImagingCore',
    ],
    'vtkmodules.vtkImagingGeneral': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkImagingCore',
    ],
    'vtkmodules.vtkImagingHybrid': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkImagingMath': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkImagingMorphological': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkImagingCore',
        'vtkmodules.vtkImagingGeneral',
    ],
    'vtkmodules.vtkImagingOpenGL2': [
        'vtkmodules.vtkImagingGeneral',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkImagingSources': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkImagingStatistics': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkImagingStencil': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkImagingCore',
    ],
    'vtkmodules.vtkInfovisCore': [
        'vtkmodules.vtkCommonColor',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkImagingSources',
        'vtkmodules.vtkIOImage',
        'vtkmodules.vtkRenderingFreeType',
    ],
    'vtkmodules.vtkInfovisLayout': [
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkInteractionImage': [
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkInteractionStyle': [
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkInteractionWidgets': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkFiltersGeneral',
        'vtkmodules.vtkFiltersSources',
        'vtkmodules.vtkRenderingContext2D',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkPythonContext2D': [
        'vtkmodules.vtkRenderingContext2D',
    ],
    'vtkmodules.vtkRenderingAnnotation': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingCellGrid': [
        'vtkmodules.vtkFiltersCellGrid',
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkRenderingContext2D': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingContextOpenGL2': [
        'vtkmodules.vtkRenderingContext2D',
        'vtkmodules.vtkRenderingFreeType',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkRenderingCore': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
    ],
    'vtkmodules.vtkRenderingExternal': [
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkRenderingFreeType': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingGridAxes': [
        'vtkmodules.vtkChartsCore',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingGL2PSOpenGL2': [
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkRenderingHyperTreeGrid': [
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingImage': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingLICOpenGL2': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkRenderingLOD': [
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingLabel': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingFreeType',
    ],
    'vtkmodules.vtkRenderingMatplotlib': [
        'vtkmodules.vtkRenderingFreeType',
    ],
    'vtkmodules.vtkRenderingOpenGL2': [
        'vtkmodules.vtkFiltersGeneral',
        'vtkmodules.vtkIOImage',
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingHyperTreeGrid',
        'vtkmodules.vtkRenderingUI',
    ],
    'vtkmodules.vtkRenderingParallel': [
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkRenderingUI': [
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingVR': [
        'vtkmodules.vtkInteractionStyle',
        'vtkmodules.vtkInteractionWidgets',
        'vtkmodules.vtkIOXMLParser',
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingOpenGL2',
        'vtkmodules.vtkRenderingVolumeOpenGL2',
        'vtkmodules.vtkRenderingVRModels',
    ],
    'vtkmodules.vtkRenderingVRModels': [
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkRenderingOpenGL2',
    ],
    'vtkmodules.vtkRenderingVolume': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkRenderingVolumeAMR': [
        'vtkmodules.vtkImagingCore',
        'vtkmodules.vtkRenderingVolume',
        'vtkmodules.vtkRenderingVolumeOpenGL2',
    ],
    'vtkmodules.vtkRenderingVolumeOpenGL2': [
        'vtkmodules.vtkImagingCore',
        'vtkmodules.vtkImagingMath',
        'vtkmodules.vtkRenderingOpenGL2',
        'vtkmodules.vtkRenderingVolume',
    ],
    'vtkmodules.vtkRenderingVtkJS': [
        'vtkmodules.vtkRenderingSceneGraph',
    ],
    'vtkmodules.vtkTestingRendering': [
        'vtkmodules.vtkImagingColor',
        'vtkmodules.vtkIOXML',
        'vtkmodules.vtkRenderingCore',
    ],
    'vtkmodules.vtkTestingSerialization': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkSerializationManager',
    ],
    'vtkmodules.vtkViewsContext2D': [
        'vtkmodules.vtkRenderingCore',
        'vtkmodules.vtkViewsCore',
    ],
    'vtkmodules.vtkViewsCore': [
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkInteractionWidgets',
    ],
    'vtkmodules.vtkViewsInfovis': [
        'vtkmodules.vtkCommonDataModel',
        'vtkmodules.vtkCommonExecutionModel',
        'vtkmodules.vtkInteractionStyle',
        'vtkmodules.vtkRenderingContext2D',
        'vtkmodules.vtkViewsCore',
    ],
    'vtkmodules.vtkWebGLExporter': [
        'vtkmodules.vtkIOExport',
    ],
}

# Additional hidden imports for vtk 9.6.0
if vtk_version is not None and vtk_version >= (9, 6, 0):
    # Additional
    _module_dependencies['vtkmodules.vtkIOImage'] += ['vtkmodules.vtkIOCore']
    _module_dependencies['vtkmodules.vtkViewsInfovis'] += ['vtkmodules.vtkChartsCore']

    # New
    _module_dependencies['vtkmodules.vtkSerializationManager'] = ['vtkmodules.vtkCommonDataModel']


def add_vtkmodules_dependencies(hook_file):
    # Find the module underlying this vtkmodules hook: change `/path/to/hook-vtkmodules.blah.py` to `vtkmodules.blah`.
    hook_name, hook_ext = os.path.splitext(os.path.basename(hook_file))
    assert hook_ext.startswith('.py')
    assert hook_name.startswith('hook-')
    module_name = hook_name[5:]

    # Look up the list of hidden imports.
    return ["vtkmodules.vtkCommonCore", *_module_dependencies.get(module_name, [])]



# ------------------------------------------------------------------
# Copyright (c) 2021 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller import isolated
from PyInstaller.utils.hooks import collect_submodules

hiddenimports = collect_submodules("ffpyplayer")
binaries = []


# ffpyplayer has an internal variable that list locations of libraries it is using.
@isolated.decorate
def _get_ffpyplayer_binary_dirs():
    import ffpyplayer
    return ffpyplayer.dep_bins


for binary_dir in _get_ffpyplayer_binary_dirs():
    binaries += [(binary_dir, '.')]  # Copy DLLs from these locations into top-level application directory.



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.utils.hooks import collect_data_files, collect_submodules

# Collect data files in frictionless/assets
datas = collect_data_files('frictionless')

# Collect modules from `frictionless.plugins` (programmatic imports).
hiddenimports = collect_submodules('frictionless.plugins')



# ------------------------------------------------------------------
# Copyright (c) 2022 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.compat import is_py310
from PyInstaller.utils.hooks import collect_submodules, collect_data_files, is_module_satisfies

# Collect core plugins.
hiddenimports = collect_submodules('hydra._internal.core_plugins')

# Hydra's plugin manager (`hydra.core.plugins.Plugins`) uses PEP-302 `find_module` / `load_module`, which has been
# deprecated since python 3.4, and has been removed from PyInstaller's frozen importer in PyInstaller 5.8. For python
# 3.10 and newer, they implemented new codepath that uses `find_spec`, but for earlier python versions, they opted to
# keep using the old codepath.
#
# See: https://github.com/facebookresearch/hydra/pull/2531
#
# To work around the incompatibility with PyInstaller >= 5.8 when using python < 3.10, force collection of plugins as
# source .py files. This way, they end up handled by python's built-in finder/importer instead of PyInstaller's
# frozen importer.
if not is_py310 and is_module_satisfies("PyInstaller >= 5.8"):
    module_collection_mode = {
        'hydra._internal.core_plugins': 'py',
        'hydra_plugins': 'py',
    }

# Collect package's data files, such as default configuration files.
datas = collect_data_files('hydra')



# ------------------------------------------------------------------
# Copyright (c) 2024 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

# Hook for https://github.com/IdentityPython/pysaml2
from PyInstaller.utils.hooks import collect_data_files, copy_metadata, collect_submodules

datas = copy_metadata("pysaml2")

# The library contains a bunch of XSD schemas that are loaded by the code:
# https://github.com/IdentityPython/pysaml2/blob/7cb4f09dce87a7e8098b9c7552ebab8bc77bc896/src/saml2/xml/schema/__init__.py#L23
# On the other hand, runtime tools are not needed.
datas += collect_data_files("saml2", excludes=["**/tools"])

# Submodules are loaded dynamically by:
# https://github.com/IdentityPython/pysaml2/blob/7cb4f09dce87a7e8098b9c7552ebab8bc77bc896/src/saml2/attribute_converter.py#L52
hiddenimports = collect_submodules("saml2.attributemaps")



#-----------------------------------------------------------------------------
# Copyright (c) 2005-2020, PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
#-----------------------------------------------------------------------------
"""
PyWin32 package 'win32com' extends its __path__ attribute with win32comext
directory and thus PyInstaller is not able to find modules in it. For example
module 'win32com.shell' is in reality 'win32comext.shell'.

>>> win32com.__path__
['win32com', 'C:\\Python27\\Lib\\site-packages\\win32comext']

"""

import os

from PyInstaller import compat
from PyInstaller import isolated
from PyInstaller.utils.hooks import logger


@isolated.decorate
def _get_win32com_file():
    try:
        import win32com
        return win32com.__file__
    except Exception:
        return None


def pre_safe_import_module(api):
    if not compat.is_win or compat.is_cygwin:
        return

    win32com_file = _get_win32com_file()
    if not win32com_file:
        logger.debug('win32com: module not available')
        return  # win32com unavailable
    win32com_dir = os.path.dirname(win32com_file)
    comext_dir = os.path.join(os.path.dirname(win32com_dir), 'win32comext')
    logger.debug('win32com: extending __path__ with dir %r' % comext_dir)
    # Append the __path__ where PyInstaller will look for 'win32com' modules.'
    api.append_package_path(comext_dir)



# ------------------------------------------------------------------
# Copyright (c) 2020 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------
from setuptools import setup, Command
import os
import datetime

DIR = os.path.dirname(__file__)


class BumpVersion(Command):
    """Bump the package version in the source files."""
    description = 'Bump the version in all registered files.'
    user_options = [
        ('major', None,
         'Bump the major (leftmost) version number. If specified, build/minor version numbers will be set to 0.'),
        ('minor', None, 'Bump the minor (middle) version number. If specified, the build NO will be set to 0.'),
        ('build', None, 'Bump the build (rightmost) version number. (Default)')
    ]

    def get_version_tuple(self, str_ver):
        return list(int(x) for x in str_ver.split('.'))

    def initialize_options(self):
        self.major = False
        self.minor = False
        self.build = True

    def finalize_options(self):
        if self.major:
            self.major = True
            self.minor = False
            self.build = False
        elif self.minor:
            self.major = False
            self.minor = True
            self.build = False
        else:
            self.major = False
            self.minor = False
            self.build = True

    def run(self):
        import re
        # REGEX:
        #  [0-9]+  - First part of version: [4].0.1
        #  [.][0-9]*  - Second part of version: 4[.0].1
        #  [.]*[0-9]*  - Third - and optional - part of version
        #  [^.]$  - The entire string must not end with a dot
        version_regex = re.compile('[0-9]+[.][0-9]*[.]*[0-9]*[^.]$')

        # List of ABSOLUTE file paths of files to bump
        files = [
            os.path.abspath(os.path.join(DIR, '_pyinstaller_hooks_contrib/__init__.py'))
        ]
        for file in files:
            old_file = open(file).readlines()
            changed = False

            for i in range(len(old_file)):
                # Get rid of line endings, if they exist
                line = old_file[i].replace('\n', '')

                # If the line starts with version, try and bump it
                if line.startswith('__version__'):
                    print('Line {ln} in {file} appears to be a version. Attempting to bump...'.format(ln=i, file=file))
                    line = line.split(' = ')[1].replace(' ', '').replace("'", '')
                    m = version_regex.match(line)
                    if m:
                        # Convert a "valid" version to a tuple of ints
                        ver = self.get_version_tuple(m.string)
                        print(ver)

                        # If the tuple isn't valid - not len(3) - then it's not a valid version
                        if len(ver) not in (2, 3):
                            print('Invalid version number. Skipping...')
                            continue

                        if len(ver) == 3:
                            if self.major:
                                ver[0] += 1
                                ver[1] = 0
                                ver[2] = 0

                            elif self.minor:
                                ver[1] += 1
                                ver[2] = 0
                            else:
                                ver[2] += 1
                        else:
                            if datetime.datetime.now().year != ver[0]:
                                ver[0] = datetime.datetime.now().year
                                ver[1] = 0
                            else:
                                ver[1] += 1

                        ver = '.'.join(str(x) for x in ver)

                        old_file[i] = old_file[i].replace(m.string, ver)
                        print('Version bumped from {} to {}.'.format(m.string, ver))
                        changed = True
                    else:
                        print('No version found - {file}:{ln}'.format(ln=i, file=file))

            if changed:
                # Write the changes to the file
                with open(file, 'w') as f:
                    f.writelines(old_file)
                # And print that to the console.
                print('Changes written to {}'.format(file))


setup(
    setup_requires="setuptools >= 30.3.0",
    entry_points={
        'pyinstaller40': [
            'hook-dirs = _pyinstaller_hooks_contrib:get_hook_dirs',
        ]
    },
    cmdclass={
        'bump': BumpVersion
    },
    long_description_content_type='text/markdown'
)



# ------------------------------------------------------------------
# Copyright (c) 2026 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.utils.hooks import collect_submodules

# The .ddgs module is lazily loaded, and is discovered only because PyInstaller currently analyzes imports under
# `if TYPE_CHECKING`. See: https://github.com/deedy5/ddgs/blob/v9.10.0/ddgs/__init__.py#L14-L15
hiddenimports = ['ddgs.ddgs']

# The engines/backends are discovered via `pkgutil.iter_modules()`, so we need to ensure they are collected.
hiddenimports += collect_submodules('ddgs.engines')



# ------------------------------------------------------------------
# Copyright (c) 2026 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------
from _pyinstaller_hooks_contrib.utils.mypy import find_mypyc_module_for_dist

# pytokens >= 0.4.0 uses `mypy`, and includes a top-level module with dynamically-generated name prefix;
# for example, `fd7dcdb10166ebd4db98__mypyc`.
hiddenimports = find_mypyc_module_for_dist('pytokens')



# ------------------------------------------------------------------
# Copyright (c) 2025 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------
from PyInstaller.utils.hooks import collect_submodules, collect_data_files

# These are all imported from cythonized extensions.
hiddenimports = [
    'json',
    'platform',
    'click',
    'mypy_extensions',
    'pathspec',
    '_black_version',
    'platformdirs',
    *collect_submodules('black'),
    # blib2to3.pytree, blib2to3.pygen, various submodules from blib2to3.pgen2; best to just collect all submodules.
    *collect_submodules('blib2to3'),
]

# Ensure that `black/resources/black.schema.json` is collected, in case someone tries to call `black.schema.get_schema`.
datas = collect_data_files('black')



# ------------------------------------------------------------------
# Copyright (c) 2020 PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
# ------------------------------------------------------------------

from PyInstaller.utils.hooks import is_module_satisfies

# sklearn.cluster in scikit-learn 0.23.x has a hidden import of
# threadpoolctl
if is_module_satisfies("scikit_learn >= 0.23"):
    hiddenimports = ['threadpoolctl', ]



#-----------------------------------------------------------------------------
# Copyright (c) 2025, PyInstaller Development Team.
#
# This file is distributed under the terms of the GNU General Public
# License (version 2.0 or later).
#
# The full license is available in LICENSE, distributed with
# this software.
#
# SPDX-License-Identifier: GPL-2.0-or-later
#-----------------------------------------------------------------------------

from PyInstaller.utils.hooks import collect_data_files, collect_submodules

# Collect submodules of distributed.http, many of which are imported indirectly.
hiddenimports = collect_submodules("distributed.http")

# Collect data files (distributed.yaml, distributed-schema.yaml, templates).
datas = collect_data_files("distributed")

# `distributed.dashboard.components.scheduler` attempts to refer to data files relative to its parent directory, but
# with non-normalized '..' elements in the path (e.g., `_MEIPASS/distributed/dashboard/components/../theme.yaml`). On
# POSIX systems, such paths are treated as non-existent if a component does not exist, even if the file exists at the
# normalized location (i.e., if `_MEIPASS/distributed/dashboard/theme.yaml` file exists but
# `_MEIPASS/distributed/dashboard/components` directory does not). As a work around, collect source .py files from
# `distributed.dashboard.components` to ensure existence of the `components` directory.
module_collection_mode = {
    'distributed.dashboard.components': 'pyz+py',
}
